package jogl.bug.test;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import com.jogamp.opengl.util.Animator;
import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureCoords;
import com.jogamp.opengl.util.texture.awt.AWTTextureIO;

// =========================================================
// javax.media -> com.jogamp
// =========================================================
import static com.jogamp.opengl.GL.GL_LINEAR;
import static com.jogamp.opengl.GL.GL_TEXTURE_2D;
import static com.jogamp.opengl.GL.GL_TEXTURE_MAG_FILTER;
import static com.jogamp.opengl.GL.GL_TEXTURE_MIN_FILTER;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.awt.GLCanvas;

// =========================================================
// Open this block if you change jogl lib to 2.2.4
// =========================================================
// import static javax.media.opengl.GL.GL_LINEAR;
// import static javax.media.opengl.GL.GL_TEXTURE_2D;
// import static javax.media.opengl.GL.GL_TEXTURE_MAG_FILTER;
// import static javax.media.opengl.GL.GL_TEXTURE_MIN_FILTER;
// import javax.media.opengl.GL2;
// import javax.media.opengl.GLAutoDrawable;
// import javax.media.opengl.GLEventListener;
// import javax.media.opengl.awt.GLCanvas;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/**
 * @author alicana
 * 
 */
public class Bug1157_Test extends GLCanvas implements GLEventListener {

	private static final long serialVersionUID = 5766742074697585883L;

	private float textureTop, textureBottom, textureLeft, textureRight;

	// ===================================================
	// Load texture
	// ===================================================

	private void loadTexture(GL2 gl) {

		BufferedImage img = null;
		URL imgUrl = null;

		imgUrl = Bug1157_Test.class.getResource("/images/jogl_logo.png");

		try {
			img = ImageIO.read(imgUrl);
		} catch (IOException e) {
			e.printStackTrace();
		}

		BufferedImage argbImg = new BufferedImage(img.getWidth(), img.getHeight(), BufferedImage.TYPE_INT_ARGB_PRE);
		Graphics2D g = argbImg.createGraphics();
		g.drawImage(img, 0, 0, null);

		Texture texture = AWTTextureIO.newTexture(gl.getGLProfile(), argbImg, false);

		gl.glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		gl.glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

		TextureCoords textureCoords = texture.getImageTexCoords();
		textureTop = textureCoords.top();
		textureBottom = textureCoords.bottom();
		textureLeft = textureCoords.left();
		textureRight = textureCoords.right();

		texture.enable(gl);
		texture.bind(gl);

		gl.glBegin(GL2.GL_QUADS);
		gl.glTexCoord2f(textureLeft, textureBottom);
		gl.glVertex3f(0.0f, 0.0f, 0.0f);
		gl.glTexCoord2f(textureRight, textureBottom);
		gl.glVertex3f(100.0f, 0.0f, 0.0f);
		gl.glTexCoord2f(textureRight, textureTop);
		gl.glVertex3f(100.0f, 100.0f, 0.0f);
		gl.glTexCoord2f(textureLeft, textureTop);
		gl.glVertex3f(0.0f, 100.0f, 0.0f);
		gl.glEnd();

	}

	// ===================================================
	// OpenGL Callback Functions
	// ===================================================
	public void init(GLAutoDrawable drawable) {
		GL2 gl = drawable.getGL().getGL2();

		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepth(1.0f);
		gl.glEnable(GL2.GL_DEPTH_TEST);
		gl.glDepthFunc(GL2.GL_LEQUAL);
		gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL2.GL_NICEST);
		gl.glShadeModel(GL2.GL_SMOOTH);

	}

	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
		GL2 gl = drawable.getGL().getGL2();

		gl.glViewport(0, 0, width, height);

		gl.glMatrixMode(GL2.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glOrthof(0, 100, 0, 100, -1, 1);

		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
	}

	public void display(GLAutoDrawable drawable) {

		GL2 gl = drawable.getGL().getGL2();

		gl.glClear(GL2.GL_COLOR_BUFFER_BIT | GL2.GL_DEPTH_BUFFER_BIT);
		gl.glLoadIdentity();

		loadTexture(gl);

		if (System.currentTimeMillis() - lastTimeInMillis > 1000) {
			System.out.println("FPS: " + fpsCounter);

			fpsCounter = 0;
			lastTimeInMillis = System.currentTimeMillis();
		}
		fpsCounter++;

		swapBuffers();
	}

	public void dispose(GLAutoDrawable drawable) {
	}

	// ===================================================
	// Main
	// ===================================================

	private static String TITLE = "JOGL 2.0 Setup (GLCanvas)";
	private static final int CANVAS_WIDTH = 320;
	private static final int CANVAS_HEIGHT = 240;
	private int fpsCounter = 0;
	private long lastTimeInMillis = System.currentTimeMillis();

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				GLCanvas canvas = new Bug1157_Test();
				canvas.setPreferredSize(new Dimension(CANVAS_WIDTH, CANVAS_HEIGHT));
				canvas.setAutoSwapBufferMode(false);

				final Animator animator = new Animator(canvas);
				animator.setRunAsFastAsPossible(false);

				final JFrame frame = new JFrame();
				frame.getContentPane().add(canvas);
				frame.addWindowListener(new WindowAdapter() {
					@Override
					public void windowClosing(WindowEvent e) {
						new Thread() {
							@Override
							public void run() {
								if (animator.isStarted())
									animator.stop();
								System.exit(0);
							}
						}.start();
					}
				});
				frame.setTitle(TITLE);
				frame.pack();
				frame.setVisible(true);
				animator.start();
			}
		});
	}

	public Bug1157_Test() {
		this.addGLEventListener(this);
	}

}